# -*- coding: utf-8 -*-
"""

Dubins TSP with Neighborhoods (DTSPN)

@author: P.Vana & P.Cizek
"""
import sys
import os
import numpy as np
import math
import matplotlib.pyplot as plt

from invoke_LKH import *

#import dubins
from DubinsManeuver import DubinsManeuver as dubins

def dist_euclidean_squared(coord1, coord2):
    (x1, y1) = coord1
    (x2, y2) = coord2
    return (x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2)

def dist_euclidean(coord1, coord2):
    return math.sqrt(dist_euclidean_squared(coord1, coord2))

class DTSPNSolver:
    def __init__(self):
        pass

    # compute the shortest sequence based on the distance matrix (self.distances)
    def compute_TSP_sequence(self, start_idx = None, end_idx = None):
        n = len(self.distances)
        
        if start_idx != None and end_idx != None:
            # TODO - insert your code here
            print ("TODO")
            M = n * np.max(np.max(self.distances))
            for i in range(n):
                self.distances[i, start_idx] = M
                self.distances[end_idx, i] = M
            self.distances[end_idx, start_idx] = 0
        
        fname_tsp = "problem"
        user_comment = "a comment by the user"
        writeTSPLIBfile_FE(fname_tsp, self.distances,user_comment)
        run_LKHsolver_cmd(fname_tsp)
        sequence = read_LKHresult_cmd(fname_tsp)
        return sequence
    
    # compute the shortest tour based on the distance matrix (self.distances)
    def compute_TSP_tour(self, start_idx = None, end_idx = None):
        n = len(self.distances)
        sequence = self.compute_TSP_sequence(start_idx=start_idx, end_idx=end_idx)
        
        if start_idx != None and end_idx != None:
            # TODO - reconstruct only the open-loop
            path = []
            for a in range(start_idx,start_idx+n-1):
                b = (a+1) % n
                a_idx = sequence[a]
                b_idx = sequence[b]
                actual_path = self.paths[(a_idx,b_idx)]
                path = path + actual_path
        else:
            path = []
            for a in range(0,n):
                b = (a+1) % n
                a_idx = sequence[a]
                b_idx = sequence[b]
                actual_path = self.paths[(a_idx,b_idx)]
                path = path + actual_path
        
        return path
        
    
    def plan_tour_etsp(self, goals):
        n = len(goals)
        self.distances = np.zeros((n,n))	
        self.paths = {}

        # find path between each pair of goals (a,b)
        for a in range(0,n): 
            for b in range(0,n):
                g1 = goals[a]
                g2 = goals[b]
                if a != b:
                    # store distance
                    self.distances[a][b] = dist_euclidean(g1, g2) 
                    # store path
                    self.paths[(a,b)] = [g1, g2]

        return self.compute_TSP_tour()

    def plan_tour_decoupled(self, goals, sensing_radius, turning_radius):
        n = len(goals)
        self.distances = np.zeros((n,n))	
        self.paths = {}

        # find path between each pair of goals (a,b)
        for a in range(0,n): 
            for b in range(0,n):
                g1 = goals[a]
                g2 = goals[b]
                if a != b:
                    # store distance
                    self.distances[a][b] = dist_euclidean(g1, g2) 
        
        '''
        TODO - homework
            1) Sample the configurations ih the goal areas
            2) Find the shortest tour
            3) Return the final tour as the points samples (step = 0.01 * radius)
        '''

        sequence = self.compute_TSP_sequence()

        path = []
        for a in range(0,n):
            b = (a+1) % n
            g1 = goals[sequence[a]]
            g2 = goals[sequence[b]]

            start = (g1[0], g1[1], 1)
            end = (g2[0], g2[1], 2)
            step_size = 0.01 * turning_radius
            dubins_path = dubins.shortest_path(start, end, turning_radius)
            configurations, _ = dubins_path.sample_many(step_size)
            path = path + configurations

        return path
    
    
     
     
    
